package models

import (
	"errors"
	"eta/eta_index_lib/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"strconv"
	"strings"
	"time"
)

// AddCalculatePhaseShift 期数移位
func AddCalculatePhaseShift(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string) (edbInfo *EdbInfo, err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}

	defer func() {
		if err != nil {
			fmt.Println("AddCalculatePhaseShift,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	// 前端那边让后端处理。。。
	if req.Frequency == "日度" && req.MoveFrequency == "" {
		return nil, errors.New("日度指标，移动频率不能为空")
	}
	switch req.Frequency {
	case "周度":
		req.MoveFrequency = "周"
	case "旬度":
		req.MoveFrequency = "旬"
	case "月度":
		req.MoveFrequency = "月"
	case "季度":
		req.MoveFrequency = "季"
	case "半年度":
		req.MoveFrequency = "半年"
	case "年度":
		req.MoveFrequency = "年"
	}

	if req.EdbInfoId <= 0 {
		edbInfo = new(EdbInfo)
		edbInfo.Source = utils.DATA_SOURCE_CALCULATE_PHASE_SHIFT
		edbInfo.SourceName = "期数移位"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbNameEn = req.EdbName
		edbInfo.UnitEn = req.Unit
		edbInfo.EdbType = 2
		edbInfo.Sort = GetAddEdbMaxSortByClassifyId(req.ClassifyId, utils.EDB_INFO_TYPE)
		edbInfo.MoveType = req.MoveType
		edbInfo.MoveFrequency = req.MoveFrequency
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			err = tmpErr
			return
		}
		edbInfo.EdbInfoId = int(newEdbInfoId)
		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfo.EdbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			calculateMappingItem.FromSubSource = edbInfo.SubSource
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	} else {
		edbInfo, err = GetEdbInfoById(req.EdbInfoId)
		if err != nil {
			return
		}
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_PHASE_SHIFT, utils.DATA_SUB_SOURCE_EDB)
		fmt.Println("dataTableName:" + dataTableName)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
		if err != nil {
			return
		}
	}

	formulaInt, _ := strconv.Atoi(req.Formula)

	//计算数据
	err = refreshAllCalculatePhaseShift(to, edbInfo.EdbInfoId, edbInfo.Source, edbInfo.SubSource, formulaInt, edbInfo.MoveType, fromEdbInfo, edbInfo.EdbCode, "", "", edbInfo.MoveFrequency)

	return
}

// EditCalculatePhaseShift 修改期数移位
func EditCalculatePhaseShift(edbInfo *EdbInfo, req *EdbInfoCalculateBatchEditReq, fromEdbInfo *EdbInfo) (err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}

	defer func() {
		if err != nil {
			fmt.Println("EditCalculatePhaseShift,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	switch req.Frequency {
	case "周度":
		req.MoveFrequency = "周"
	case "旬度":
		req.MoveFrequency = "旬"
	case "月度":
		req.MoveFrequency = "月"
	case "季度":
		req.MoveFrequency = "季"
	case "半年度":
		req.MoveFrequency = "半年"
	case "年度":
		req.MoveFrequency = "年"
	}

	oldEdbInfo := *edbInfo //旧的指标信息

	//修改指标信息
	edbInfo.EdbName = req.EdbName
	edbInfo.EdbNameSource = req.EdbName
	edbInfo.Frequency = req.Frequency
	edbInfo.Unit = req.Unit
	edbInfo.ClassifyId = req.ClassifyId
	edbInfo.MoveType = req.MoveType
	edbInfo.MoveFrequency = req.MoveFrequency
	edbInfo.CalculateFormula = req.Formula
	edbInfo.EdbNameEn = req.EdbNameEn
	edbInfo.UnitEn = req.UnitEn
	edbInfo.ModifyTime = time.Now()
	_, err = to.Update(edbInfo, "EdbName", "EdbNameSource", "Frequency", "Unit", "ClassifyId", "MoveType", "MoveFrequency", "CalculateFormula", "ModifyTime", "EdbNameEn", "UnitEn")
	if err != nil {
		return
	}

	//判断计算指标是否被更换
	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? AND from_edb_info_id=? "
	existPars = append(existPars, edbInfo.EdbInfoId, req.FromEdbInfoId)

	count, err := GetEdbInfoCalculateCountByCondition(existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}

	if count <= 0 || req.MoveType != oldEdbInfo.MoveType || req.MoveFrequency != oldEdbInfo.MoveFrequency || req.Formula != oldEdbInfo.CalculateFormula {
		//如果是依赖指标变更，那么需要删除对应的关联指标，并添加新的关系
		if count <= 0 {
			//删除，计算指标关联的，基础指标的关联关系
			sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
			_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
			if err != nil {
				err = errors.New("删除计算指标关联关系失败,Err:" + err.Error())
				return
			}
			//关联关系
			{
				calculateMappingItem := &EdbInfoCalculateMapping{
					EdbInfoCalculateMappingId: 0,
					EdbInfoId:                 edbInfo.EdbInfoId,
					Source:                    utils.DATA_SOURCE_CALCULATE_PHASE_SHIFT,
					SourceName:                "期数移位",
					EdbCode:                   edbInfo.EdbCode,
					FromEdbInfoId:             fromEdbInfo.EdbInfoId,
					FromEdbCode:               fromEdbInfo.EdbCode,
					FromEdbName:               fromEdbInfo.EdbName,
					FromSource:                fromEdbInfo.Source,
					FromSourceName:            fromEdbInfo.SourceName,
					FromTag:                   "",
					Sort:                      1,
					CreateTime:                time.Now(),
					ModifyTime:                time.Now(),
					FromSubSource:             fromEdbInfo.SubSource,
				}
				_, err = to.Insert(calculateMappingItem)
				if err != nil {
					return
				}
			}
		}

		//清空原有数据
		sql := ` DELETE FROM edb_data_calculate_phase_shift WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
		if err != nil {
			return
		}

		//计算数据
		formulaInt, _ := strconv.Atoi(req.Formula)
		err = refreshAllCalculatePhaseShift(to, edbInfo.EdbInfoId, edbInfo.Source, edbInfo.SubSource, formulaInt, edbInfo.MoveType, fromEdbInfo, edbInfo.EdbCode, "", "", edbInfo.MoveFrequency)
	}
	return
}

// RefreshAllCalculatePhaseShift 刷新所有时间移位数据
func RefreshAllCalculatePhaseShift(edbInfoId, source, subSource, formulaInt, moveType int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate, moveFrequency string) (err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("RefreshAllCalculatePhaseShift,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//计算数据
	err = refreshAllCalculatePhaseShift(to, edbInfoId, source, subSource, formulaInt, moveType, fromEdbInfo, edbCode, startDate, endDate, moveFrequency)

	return
}

// refreshAllCalculatePhaseShift 刷新所有时间移位数据
func refreshAllCalculatePhaseShift(to orm.TxOrmer, edbInfoId, source, subSource, formulaInt, moveType int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate, moveFrequency string) (err error) {
	edbInfoIdStr := strconv.Itoa(edbInfoId)

	//计算数据
	dataList, err := GetEdbDataListAllByTo(to, fromEdbInfo.Source, fromEdbInfo.SubSource, FindEdbDataListAllCond{
		EdbInfoId:         fromEdbInfo.EdbInfoId,
		StartDataTime:     startDate,
		StartDataTimeCond: ">=",
	}, 0)
	if err != nil {
		return err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}
	fmt.Println("source:", source)
	//获取指标所有数据
	existDataList := make([]*EdbData, 0)
	dataTableName := GetEdbDataTableName(source, subSource)
	fmt.Println("dataTableName:", dataTableName)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return err
	}

	existDataMap := make(map[string]string)
	removeDateMap := make(map[string]string)
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v.Value
		removeDateMap[v.DataTime] = ``
	}
	fmt.Println("existDataMap:", existDataMap)
	addSql := ` INSERT INTO edb_data_calculate_phase_shift(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool

	resultMap := make(map[string]float64)
	dataLen := len(dataList)
	var moveNum int
	for i := 0; i < dataLen; i++ {
		// step_1 如果 领先/滞后 之后时间key存在，将该key为目标key，填充
		currentIndex := dataList[i]

		// 领先
		if moveType != 2 {
			periods := dataLen - i + formulaInt - 1
			if periods < dataLen {
				newIndex := dataList[dataLen-periods-1]
				resultMap[newIndex.DataTime] = currentIndex.Value
			} else {
				moveNum = formulaInt - i

				// 新数据须根据频度补充key
				currentDate, _ := time.ParseInLocation(utils.FormatDate, dataList[0].DataTime, time.Local)

				shiftDay := CalculateIntervalDays(moveFrequency, moveNum, currentDate, resultMap, moveType)

				var newDate time.Time
				if moveFrequency == "年" {
					newDate = currentDate.AddDate(moveNum, 0, 0)
				} else {
					newDate = currentDate.AddDate(0, 0, shiftDay)
				}

				format := newDate.Format(utils.FormatDate)
				resultMap[format] = currentIndex.Value
			}
		} else {
			// 滞后
			periods := dataLen - i - formulaInt
			if periods > 0 {
				newIndex := dataList[dataLen-periods]
				resultMap[newIndex.DataTime] = currentIndex.Value
			} else {
				moveNum = formulaInt + 1 - (dataLen - i)
				// 新数据须根据频度补充key
				currentDate, _ := time.ParseInLocation(utils.FormatDate, dataList[dataLen-1].DataTime, time.Local)

				shiftDay := CalculateIntervalDays(moveFrequency, moveNum, currentDate, resultMap, moveType)

				var newDate time.Time
				if moveFrequency == "年" {
					newDate = currentDate.AddDate(-moveNum, 0, 0)
				} else {
					newDate = currentDate.AddDate(0, 0, -shiftDay)
				}

				format := newDate.Format(utils.FormatDate)
				resultMap[format] = currentIndex.Value
			}
		}
	}

	for key, value := range resultMap {
		currentDate, _ := time.ParseInLocation(utils.FormatDate, key, time.Local)
		timestamp := currentDate.UnixNano() / 1e6
		timestampStr := fmt.Sprintf("%d", timestamp)
		addSql += GetAddSql(edbInfoIdStr, edbCode, key, timestampStr, fmt.Sprintf("%f", value))
		isAdd = true
	}

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return
		}
	}

	return
}

func CalculateIntervalDays(moveFrequency string, formulaInt int, baseDate time.Time, resultMap map[string]float64, moveType int) int {
	var shiftDay int
	switch moveFrequency {
	case "自然日":
		shiftDay = formulaInt
	case "交易日":
		shiftDay = utils.CalculateTradingDays(baseDate, formulaInt, resultMap, moveType)
	case "周":
		shiftDay = formulaInt * 7
	case "旬":
		shiftDay = utils.CalculateDekadTime(baseDate, formulaInt, moveType)
	case "月":
		shiftDay = utils.CalculateEndOfMonth(baseDate, formulaInt, moveType)
	case "季":
		shiftDay = utils.CalculateEndOfQuarter(baseDate, formulaInt, moveType)
	case "半年":
		shiftDay = utils.CalculateEndOfHalfYear(baseDate, formulaInt, moveType)
	case "年":
		shiftDay = utils.CalculateEndOfYear(baseDate, formulaInt, moveType)
	default:
		shiftDay = formulaInt
	}
	return shiftDay
}
