package models

import (
	"errors"
	"eta/eta_index_lib/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"strconv"
	"strings"
	"time"
)

// SavePredictCalculatePhaseShift 期数移位
func SavePredictCalculatePhaseShift(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName, lang string) (edbInfo *EdbInfo, latestDateStr string, latestValue float64, err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}

	defer func() {
		if err != nil {
			fmt.Println("SavePredictCalculateTimeShift,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	// 前端那边让后端处理。。。
	if req.Frequency == "日度" && req.MoveFrequency == "" {
		return nil, "", 0, errors.New("日度指标，移动频率不能为空")
	}
	switch req.Frequency {
	case "周度":
		req.MoveFrequency = "周"
	case "旬度":
		req.MoveFrequency = "旬"
	case "月度":
		req.MoveFrequency = "月"
	case "季度":
		req.MoveFrequency = "季"
	case "半年度":
		req.MoveFrequency = "半年"
	case "年度":
		req.MoveFrequency = "年"
	}

	if req.EdbInfoId <= 0 {
		edbInfo = new(EdbInfo)
		edbInfo.EdbInfoType = 1
		edbInfo.Source = utils.DATA_SOURCE_PREDICT_CALCULATE_PHASE_SHIFT
		edbInfo.SourceName = "预测期数位移"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbType = 2
		edbInfo.MoveType = req.MoveType
		edbInfo.MoveFrequency = req.MoveFrequency
		edbInfo.EdbNameEn = req.EdbName
		edbInfo.UnitEn = req.Unit
		edbInfo.Sort = GetAddEdbMaxSortByClassifyId(req.ClassifyId, utils.PREDICT_EDB_INFO_TYPE)
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			err = tmpErr
			return
		}
		edbInfo.EdbInfoId = int(newEdbInfoId)
		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfo.EdbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	} else {
		edbInfo, err = GetEdbInfoById(req.EdbInfoId)
		if err != nil {
			return
		}
		latestDateStr = edbInfo.LatestDate
		latestValue = edbInfo.LatestValue
		oldEdbInfo := *edbInfo //旧的指标信息

		//修改指标信息
		switch lang {
		case utils.EnLangVersion:
			edbInfo.EdbNameEn = req.EdbName
			edbInfo.UnitEn = req.Unit
		default:
			edbInfo.EdbName = req.EdbName
			edbInfo.Unit = req.Unit
			edbInfo.EdbNameSource = req.EdbName
		}
		edbInfo.Frequency = req.Frequency
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.MoveType = req.MoveType
		edbInfo.MoveFrequency = req.MoveFrequency
		edbInfo.CalculateFormula = req.Formula
		edbInfo.ModifyTime = time.Now()
		_, err = to.Update(edbInfo, "EdbName", "EdbNameSource", "Frequency", "Unit", "ClassifyId", "MoveType", "MoveFrequency", "CalculateFormula", "ModifyTime", "EdbNameEn", "UnitEn")
		if err != nil {
			return
		}

		//判断计算指标是否被更换
		var existCondition string
		var existPars []interface{}
		existCondition += " AND edb_info_id=? AND from_edb_info_id=? "
		existPars = append(existPars, edbInfo.EdbInfoId, req.FromEdbInfoId)

		var count int
		count, err = GetEdbInfoCalculateCountByCondition(existCondition, existPars)
		if err != nil {
			err = errors.New("判断指标是否改变失败,Err:" + err.Error())
			return
		}

		if count <= 0 || req.MoveType != oldEdbInfo.MoveType || req.MoveFrequency != oldEdbInfo.MoveFrequency || req.Formula != oldEdbInfo.CalculateFormula {
			//如果是依赖指标变更，那么需要删除对应的关联指标，并添加新的关系
			if count <= 0 {
				//删除，计算指标关联的，基础指标的关联关系
				sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
				_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
				if err != nil {
					err = errors.New("删除计算指标关联关系失败,Err:" + err.Error())
					return
				}
				//关联关系
				{
					calculateMappingItem := &EdbInfoCalculateMapping{
						EdbInfoCalculateMappingId: 0,
						EdbInfoId:                 edbInfo.EdbInfoId,
						Source:                    utils.DATA_SOURCE_PREDICT_CALCULATE_PHASE_SHIFT,
						SourceName:                "预测期数位移",
						EdbCode:                   edbInfo.EdbCode,
						FromEdbInfoId:             fromEdbInfo.EdbInfoId,
						FromEdbCode:               fromEdbInfo.EdbCode,
						FromEdbName:               fromEdbInfo.EdbName,
						FromSource:                fromEdbInfo.Source,
						FromSourceName:            fromEdbInfo.SourceName,
						FromTag:                   "",
						Sort:                      1,
						CreateTime:                time.Now(),
						ModifyTime:                time.Now(),
					}
					_, err = to.Insert(calculateMappingItem)
					if err != nil {
						return
					}
				}
			}

			//清空原有数据
			sql := ` DELETE FROM edb_data_predict_calculate_phase_shift WHERE edb_info_id = ? `
			_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
			if err != nil {
				return
			}
		} else {
			return
		}
	}

	formulaInt, _ := strconv.Atoi(req.Formula)

	//计算数据
	latestDateStr, latestValue, err = refreshAllPredictCalculatePhaseShift(to, edbInfo.EdbInfoId, edbInfo.Source, edbInfo.SubSource, formulaInt, edbInfo.MoveType, fromEdbInfo, edbInfo.EdbCode, "", "", edbInfo.MoveFrequency)

	return
}

// RefreshAllPredictCalculatePhaseShift 刷新所有时间移位数据
func RefreshAllPredictCalculatePhaseShift(edbInfoId, source, subSource, formulaInt, moveType int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate, moveFrequency string) (latestDateStr string, latestValue float64, err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("RefreshAllPredictCalculatePhaseShift,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//计算数据
	latestDateStr, latestValue, err = refreshAllPredictCalculatePhaseShift(to, edbInfoId, source, subSource, formulaInt, moveType, fromEdbInfo, edbCode, startDate, endDate, moveFrequency)
	return
}

// refreshAllPredictCalculatePhaseShift 刷新所有时间移位数据
func refreshAllPredictCalculatePhaseShift(to orm.TxOrmer, edbInfoId, source, subSource, formulaInt, moveType int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate, moveFrequency string) (latestDateStr string, latestValue float64, err error) {
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	dataList, err := GetPredictEdbDataListAllByStartDate(fromEdbInfo, 0, startDate)
	if err != nil {
		return
	}

	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_predict_calculate_phase_shift(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool

	resultMap := make(map[string]float64)
	dataLen := len(dataList)
	var moveNum int
	for i := 0; i < dataLen; i++ {
		// step_1 如果 领先/滞后 之后时间key存在，将该key为目标key，填充
		currentIndex := dataList[i]

		// 领先
		if moveType != 2 {
			periods := dataLen - i + formulaInt - 1
			if periods < dataLen {
				newIndex := dataList[dataLen-periods-1]
				resultMap[newIndex.DataTime] = currentIndex.Value

				if strings.Contains(currentIndex.DataTime, fromEdbInfo.LatestDate) {
					latestDateStr = newIndex.DataTime
				}
			} else {
				moveNum = formulaInt - i

				// 新数据须根据频度补充key
				currentDate, _ := time.ParseInLocation(utils.FormatDate, dataList[0].DataTime, time.Local)

				shiftDay := CalculateIntervalDays(moveFrequency, moveNum, currentDate, resultMap, moveType)

				var newDate time.Time
				if moveFrequency == "年" {
					newDate = currentDate.AddDate(moveNum, 0, 0)
				} else {
					newDate = currentDate.AddDate(0, 0, shiftDay)
				}

				format := newDate.Format(utils.FormatDate)

				if strings.Contains(currentIndex.DataTime, fromEdbInfo.LatestDate) {
					latestDateStr = format
				}
				resultMap[format] = currentIndex.Value
			}
		} else {
			// 滞后
			periods := dataLen - i - formulaInt
			if periods > 0 {
				newIndex := dataList[dataLen-periods]
				resultMap[newIndex.DataTime] = currentIndex.Value

				if strings.Contains(currentIndex.DataTime, fromEdbInfo.LatestDate) {
					latestDateStr = newIndex.DataTime
				}
			} else {
				moveNum = formulaInt + 1 - (dataLen - i)
				// 新数据须根据频度补充key
				currentDate, _ := time.ParseInLocation(utils.FormatDate, dataList[dataLen-1].DataTime, time.Local)

				shiftDay := CalculateIntervalDays(moveFrequency, moveNum, currentDate, resultMap, moveType)

				var newDate time.Time
				if moveFrequency == "年" {
					newDate = currentDate.AddDate(-moveNum, 0, 0)
				} else {
					newDate = currentDate.AddDate(0, 0, -shiftDay)
				}

				format := newDate.Format(utils.FormatDate)
				resultMap[format] = currentIndex.Value

				if strings.Contains(currentIndex.DataTime, fromEdbInfo.LatestDate) {
					latestDateStr = format
				}
			}
		}
	}

	for key, value := range resultMap {
		currentDate, _ := time.ParseInLocation(utils.FormatDate, key, time.Local)
		timestamp := currentDate.UnixNano() / 1e6
		timestampStr := fmt.Sprintf("%d", timestamp)
		addSql += GetAddSql(edbInfoIdStr, edbCode, key, timestampStr, fmt.Sprintf("%f", value))
		isAdd = true
	}

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return
		}
	}

	return
}
